/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::RobotNameService
 * @author     [Author Name] ( [Author Email] )
 * @date       2018
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "RobotNameService.h"

namespace armarx
{
    void
    RobotNameService::onInitComponent()
    {
    }

    void
    RobotNameService::onConnectComponent()
    {
    }

    void
    RobotNameService::onDisconnectComponent()
    {
    }

    void
    RobotNameService::onExitComponent()
    {
    }

    armarx::PropertyDefinitionsPtr
    RobotNameService::createPropertyDefinitions()
    {
        armarx::PropertyDefinitionsPtr defs =
            new ComponentPropertyDefinitions(getConfigIdentifier());

        return defs;
    }

    bool
    RobotNameService::registerRobotInfo(const robot_name_service::dto::RobotInfo& robot,
                                        const Ice::Current& current)
    {
        std::scoped_lock l(robotsMutex);
        ARMARX_INFO << "Register a new robot with name '" << robot.name << "' in RNS";

        if (auto it = robots.find(robot.name); it != robots.end())
        {
            ARMARX_ERROR << "The robot with name '" << robot.name
                         << "' is already registered. Ignoring it.";
            return false;
        }

        robots[robot.name].fromIce(robot);
        return true;
    }

    void
    RobotNameService::unregisterRobotInfo(const std::string& name, const Ice::Current& current)
    {
        std::scoped_lock l(robotsMutex);

        if (auto it = robots.find(name); it != robots.end())
        {
            robots.erase(it);
        }
    }

    IceUtil::Optional<robot_name_service::dto::RobotInfo>
    RobotNameService::getRobotInfo(const std::string& name, const Ice::Current& current)
    {
        std::scoped_lock l(robotsMutex);

        if (auto it = robots.find(name); it == robots.end())
        {
            return {};
        }

        return robots[name].toIce();
    }
} // namespace armarx
