#include "ClassWriter.h"

namespace armarx::armem::obj::clazz
{
    ClassWriter::ClassWriter(const std::string& p) :
        providerName(p)
    {

    }

    void ClassWriter::setProviderName(const std::string& pName)
    {
        this->providerName = pName;
    }

    bool
    ClassWriter::commitObjectClass(const armarx::armem::clazz::ObjectClass& c,
                                   const armarx::core::time::DateTime& referenceTime)
    {
        armarx::armem::arondto::ObjectClass objClassAron;
        armarx::armem::clazz::toAron(objClassAron, c);

        armarx::armem::MemoryID mid;
        mid.memoryName = properties().memoryName;
        mid.coreSegmentName = properties().coreSegmentName;
        mid.providerSegmentName = properties().providerName;
        mid.entityName = c.id.str();

        auto commit = armarx::armem::Commit();
        auto& update = commit.add();

        update.confidence = 1.0f;
        update.referencedTime = referenceTime;
        update.sentTime = armarx::core::time::DateTime::Now();
        update.entityID = mid;
        update.instancesData = {objClassAron.toAron()};

        auto res = this->memoryWriter().commit(commit);

        return res.allSuccess();
    }

    std::string
    ClassWriter::propertyPrefix() const
    {
        return "classWriter.";
    }

    client::util::SimpleWriterBase::Properties
    ClassWriter::defaultProperties() const
    {
        client::util::SimpleWriterBase::Properties p;
        p.memoryName = "Object";
        p.coreSegmentName = "Class";
        p.providerName = providerName;
        return p;
    }

} // namespace armarx::armem::obj::clazz
